@@ -901,6 +901,34 @@ meta_comp_effect_get_type (void)
   return g_enum_type_id__volatile;
 }
 
+/* enumerations from "include/screen.h" */
+#include "include/screen.h"
+
+GType
+meta_screen_corner_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { META_SCREEN_TOPLEFT, "META_SCREEN_TOPLEFT", "topleft" },
+        { META_SCREEN_TOPRIGHT, "META_SCREEN_TOPRIGHT", "topright" },
+        { META_SCREEN_BOTTOMLEFT, "META_SCREEN_BOTTOMLEFT", "bottomleft" },
+        { META_SCREEN_BOTTOMRIGHT, "META_SCREEN_BOTTOMRIGHT", "bottomright" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("MetaScreenCorner"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
+
 /* enumerations from "include/display.h" */
 #include "include/display.h"
 
